#!/bin/bash

sudo apt install make g++ libasound2-dev fonts-noto-core libharfbuzz-dev imagemagick libpam0g-dev fonts-noto-color-emoji fonts-noto-cjk fonts-noto-mono libwebsockets-dev libdrm-dev libfreetype-dev librsvg2-dev
