
// SPDX-License-Identifier: CC-BY-NC-SA-4.0
//
// Copyright (C) 2026 Bit by Bit Signal Processing LLC (https://bxbsp.com)
//
// This work is placed under the "Creative Commons Attribution
// NonCommercial ShareAlike 4.0 International" license, known
// by the shortened acronym "CC-BY-NC-SA-4.0".
//
// This work is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// A CC-BY-NC-SA-4.0 license allows you to use this work for
// noncommercial purposes so long as attribution is made to the
// original author.  Modified versions of this work may be distributed,
// but only under the same license.  For further details, see the
// Creative Commons License "CC-BY-NC-SA-4.0".
//
// You should have received a copy of the CC-BY-NC-SA-4.0 license
// along with this work. If not, see
// <https://creativecommons.org/licenses/by-nc-sa/4.0/>.
//

//

#ifndef GLOBAL_DRAW_LINE_HH
#define GLOBAL_DRAW_LINE_HH

#include "panel.hh"

void global_draw_line(panel* pan, pointf p0, pointf p1, color c);
void global_draw_line_horizontal(panel* pan, point p0i, point p1i, color c);
void global_draw_line_vertical(panel* pan, point p0i, point p1i, color c);
void global_draw_line_known_background(panel* pan, pointf p0, pointf p1, color c, color bg);


#endif
